import type { Block, BlockType } from '../types.d'

export const BLOCK_SIZE = 64 // each block is 64x64 px in size and equals 1m
export const RECIPROCAL = 1 / BLOCK_SIZE

export const STAGE_WIDTH = 20 // 20*64 = 1280 pixel wide stage
// export const STAGE_HEIGHT = ~~(STAGE_WIDTH * 0.5625) // 16:9 😎
export const STAGE_HEIGHT = 12 // 12*64 = 768 pixel high stage

export const GRAVITY = 10 // blocks per second

export const blockTypes: Record<BlockType, Block> = {
  // Transparent Blocks
  air: { type: 'air', hp: Infinity, walkable: true, transparent: true },
  cave: { type: 'cave', hp: Infinity, walkable: true, transparent: true },
  // Tree Parts
  treeCrown: { type: 'treeCrown', hp: 1, walkable: true, transparent: true, drops: 'block_leaves' },
  treeLeaves: { type: 'treeLeaves', hp: 1, walkable: true, transparent: true, drops: 'block_leaves' },
  treeTrunk: { type: 'treeTrunk', hp: 10, walkable: true, climbable: true, transparent: true, drops: 'block_wood' },
  treeRoot: { type: 'treeRoot', hp: 10, walkable: true, climbable: true, drops: 'block_wood' },
  // Opaque Natural Blocks
  grass: { type: 'grass', hp: 5, walkable: false, drops: 'block_dirt' },
  soil: { type: 'soil', hp: 5, walkable: false, drops: 'block_dirt' },
  soilGravel: { type: 'soilGravel', hp: 5, walkable: false, drops: 'block_gravel' },
  stoneGravel: { type: 'stoneGravel', hp: 10, walkable: false, drops: 'block_gravel' },
  stone: { type: 'stone', hp: 10, walkable: false, drops: 'block_stone' },
  bedrock: { type: 'bedrock', hp: 25, walkable: false, drops: 'block_stone' },
  // Built Blocks
  brickWall: { type: 'brickWall', hp: 25, walkable: false, drops: 'block_gravel' },

  torchLeft: { type: 'torchLeft', hp: 1, walkable: true, transparent: true, drops: 'fixture_torch', illumination: 1.0, color: '#FFE', fixture: true },
  torchRight: { type: 'torchRight', hp: 1, walkable: true, transparent: true, drops: 'fixture_torch', illumination: 1.0, color: '#FFE', fixture: true },
  torchCeiling: { type: 'torchCeiling', hp: 1, walkable: true, transparent: true, drops: 'fixture_torch', illumination: 1.0, color: '#FFE', fixture: true },
  torchFloor: { type: 'torchFloor', hp: 1, walkable: true, transparent: true, drops: 'fixture_torch', illumination: 1.0, color: '#FEB', fixture: true },
}

export const softTerrain: BlockType[] = [
  'grass', 'soil', 'soilGravel',
  'torchLeft', 'torchRight', 'torchCeiling', 'torchFloor',
]
export const hardTerrain: BlockType[] = ['stone', 'stoneGravel', 'bedrock', 'brickWall']

export const level = {
  treeTop: 9,
  ground: 14,
  rock: 16,
  underground: 24,
  caveMax: 250,
}

export const probability = {
  tree: 0.3,
  soilHole: 0.3,
  soilGravel: 0.2,
  stoneGravel: 0.1,
  cave: 0.5,
  fray: 0.4,
}
